﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using UHF_Reader.Public;
using System.Threading;
using System.IO;

namespace UHF_Reader
{
    public partial class Main : Form
    {
        public Main()
        {
            InitializeComponent();
        }

        private void Main_Load(object sender, EventArgs e)
        {
            string[] serials = new string[32];

            int hidCnt = PublicFunction.SearchHids(ref serials);

            if (hidCnt == 0)
            {
                return;
            }
            else
            {
                for (int i = 0; i < hidCnt; i++)
                {
                    CbCom.Items.Add(serials[i]);
                }
            }

            if (CbCom.Items.Count != 0)
            {
                CbCom.SelectedIndex = 0;
            }
            else
            {
                MessageBox.Show("No device was found");
            }

            DemoPublic.PublicDM = this;
            timer1.Interval = 200;
            CbQ.SelectedIndex = 3;
        }

        private void BtConnect_Click(object sender, EventArgs e)
        {
            if (BtConnect.Text == "Connect")
            {
                string comStr = CbCom.SelectedItem.ToString();
                if (comStr == "")
                {
                    MessageBox.Show("Please select a device");
                    return;
                }

                PublicFunction.SelectBaudAndCrc(57600, 0x01);

                if (PublicFunction.ConnectRLM(comStr))
                {
                    AddToShow("Connect success");
                    BtConnect.Text = "DisConnect";
                    DemoPublic.Enabel_flg = true;
                }
                else
                {
                    AddToShow("Connect fail");
                }
            }
            else
            {
                if (PublicFunction.DisConnectRLM())
                {
                    BtConnect.Text = "Connect";
                    DemoPublic.Enabel_flg = false;
                    AddToShow("Disconnect success");
                }
                else
                    AddToShow("Disconnect fail");
            }
        }

        private void BtInventory_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            if (BtInventory.Text == "Inventory")
            {
                timer1.Enabled = true;

                DemoPublic.bQ = (byte)CbQ.SelectedIndex;

                if (PublicFunction.Inventory(0x01, DemoPublic.bQ))
                {
                    DemoPublic.Para para = new DemoPublic.Para();

                    Object obj = (Object)para;


                    BtInventory.Text = "Stop";

                    DemoPublic.LoopEnable = true;

                    DemoPublic.EPCThread = new Thread(new ParameterizedThreadStart(PublicFunction.GetUiiThread_Timer));

                    DemoPublic.EPCThread.IsBackground = true;

                    DemoPublic.EPCThread.Start(obj);

                    AddToShow("Inventory starts");
                    AddToShow("........ ........");
                }
                else
                {
                    AddToShow("Inventory failed");
                    BtInventory.Text = "Inventory";
                }
            }
            else
            {
                timer1.Enabled = false;
                DemoPublic.LoopEnable = false;

                if (PublicFunction.Stop())
                {
                    BtInventory.Text = "Inventory";
                    AddToShow("Inventory stops");
                }
                else
                {
                    AddToShow("Stop fail");
                }

                if (DemoPublic.EPCThread != null)
                {
                    DemoPublic.EPCThread.Abort();
                }
            }
        }

        private void BtReadTid_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            if (BtReadTid.Text == "Read TID")
            {
                AddToShow("Start to read tid");

                timer1.Enabled = true;

                byte[] Uii = new byte[64];
                byte[] pwd = { 0x00, 0x00, 0x00, 0x00 };
                DemoPublic.Para para = new DemoPublic.Para();
                para.bank = 0x02;
                para.addr = 0x00;
                para.pwd = pwd;

                if (ChkBox_withuii.Checked)
                {
                    string uiiStr = TbUii.Text.ToString().Trim();

                    if (!DemoPublic.CheckDigit(uiiStr))
                    {
                        MessageBox.Show("UII must be hex string");
                        return;
                    }
                    DemoPublic.HexStringToBytes(uiiStr, Uii);
                    para.withuii = true;
                    para.uii = Uii;
                }
                else
                {
                    para.withuii = false;
                    para.uii = Uii;
                }

                Object obj = (Object)para;

                BtReadTid.Text = "Stop";

                DemoPublic.LoopEnable = true;

                DemoPublic.EPCThread = new Thread(new ParameterizedThreadStart(PublicFunction.ReadDataThread_Timer));

                DemoPublic.EPCThread.IsBackground = true;

                DemoPublic.EPCThread.Start(obj);

            }
            else
            {
                timer1.Enabled = false;
                DemoPublic.LoopEnable = false;

                BtReadTid.Text = "Read TID";
                AddToShow("Stop reading TID");

                if (DemoPublic.EPCThread != null)
                {
                    DemoPublic.EPCThread.Abort();
                }
            }
        }

        private void BtReadUser_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            if (BtReadUser.Text == "Read USER")
            {
                AddToShow("Start to read user");

                timer1.Enabled = true;

                byte[] Uii = new byte[64];
                DemoPublic.Para para = new DemoPublic.Para();
                para.bank = 0x03;
                para.addr = 0x00;
                para.pwd = new byte[4];
                if (ChkBox_withuii.Checked)
                {
                    string uiiStr = TbUii.Text.ToString().Trim();

                    if (!DemoPublic.CheckDigit(uiiStr))
                    {
                        MessageBox.Show("UII must be hex string");
                        return;
                    }
                    DemoPublic.HexStringToBytes(uiiStr, Uii);
                    para.withuii = true;
                    para.uii = Uii;
                }
                else
                {
                    para.withuii = false;
                    para.uii = Uii;
                }

                Object obj = (Object)para;

                BtReadUser.Text = "Stop";

                DemoPublic.LoopEnable = true;

                DemoPublic.EPCThread = new Thread(new ParameterizedThreadStart(PublicFunction.ReadDataThread_Timer));

                DemoPublic.EPCThread.IsBackground = true;

                DemoPublic.EPCThread.Start(obj);

            }
            else
            {
                timer1.Enabled = false;
                DemoPublic.LoopEnable = false;

                BtReadUser.Text = "Read USER";
                AddToShow("Stop reading USER");

                if (DemoPublic.EPCThread != null)
                {
                    DemoPublic.EPCThread.Abort();
                }
            }
        }

        private void BtWriteEPC_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            byte[] pwd = { 0x00, 0x00, 0x00, 0x00 };
            byte[] uii = new byte[64];
            byte[] uiilen = new byte[1];
            byte[] writedata = new byte[256];
            byte[] error = new byte[1];
            byte[] writelen = new byte[1];
            byte cnt = 0;

            if (ChkBox_withuii.Checked)
            {
                string uiiStr = TbUii.Text.ToString().Trim();

                if (!DemoPublic.CheckDigit(uiiStr))
                {
                    MessageBox.Show("UII must be hex string");
                    return;
                }
                DemoPublic.HexStringToBytes(uiiStr, uii);
            }

            string tempStr = TbData.Text.ToString().Trim();

            if (tempStr == "")
            {
                MessageBox.Show("Please input data");
                return;
            }

            string dataStr = tempStr.Replace(" ", "");
            if (RbHanzi.Checked)
            {
                dataStr = DemoPublic.HanziToUnicodeHexString(dataStr);
            }
            cnt = (byte)(dataStr.Length / 4);
            DemoPublic.HexStringToBytes(dataStr, writedata);

            if (ChkBox_withuii.Checked)
            {
                if (PublicFunction.WriteEPC(pwd, cnt, uii, writedata, error, writelen))
                {
                    System.Media.SystemSounds.Asterisk.Play();

                    AddToShow("Change EPC successfully with specified tag");
                }
                else
                {
                    AddToShow("Fail to change EPC with specified tag");
                }
            }
            else
            {
                if (PublicFunction.WriteEPCSingle(pwd, cnt, uii, uiilen, writedata, error, writelen))
                {
                    System.Media.SystemSounds.Asterisk.Play();

                    AddToShow("Change EPC successfully");
                }
                else
                {
                    AddToShow("Fail to change EPC");
                }
            }

        }

        private void BtWriteData_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            if (BtWriteData.Text == "Write USER")
            {
                AddToShow("Start to write USER");
                byte[] pwd = { 0x00, 0x00, 0x00, 0x00 };
                byte len = 0x00;
                byte[] Uii = new byte[64];
                byte[] writedata = new byte[256];

                string tempStr = TbData.Text.ToString().Trim();

                if (tempStr == "")
                {
                    MessageBox.Show("Please input data");
                    return;
                }

                string dataStr = tempStr.Replace(" ", "");
                //Console.WriteLine("dataStr:" + dataStr);
                if (RbHanzi.Checked)
                {
                    dataStr = DemoPublic.HanziToUnicodeHexString(dataStr);
                }
                //Console.WriteLine("--dataStr:"+dataStr);
                len = (byte)(dataStr.Length / 4);
                DemoPublic.HexStringToBytes(dataStr, writedata);
                //Console.WriteLine("writedata:" + writedata[0] + " " + writedata[1]);

                DemoPublic.Para para = new DemoPublic.Para();
                para.pwd = pwd;
                para.uii = Uii;
                para.len = len;
                para.writedata = writedata;

                if (ChkBox_withuii.Checked)
                {
                    string uiiStr = TbUii.Text.ToString().Trim();

                    if (!DemoPublic.CheckDigit(uiiStr))
                    {
                        MessageBox.Show("UII must be hex string");
                        return;
                    }
                    DemoPublic.HexStringToBytes(uiiStr, Uii);
                    para.withuii = true;
                    para.uii = Uii;
                }
                else
                {
                    para.withuii = false;
                    para.uii = Uii;
                }

                Object obj = (Object)para;

                timer1.Enabled = true;
                BtWriteData.Text = "Stop";

                DemoPublic.LoopEnable = true;

                DemoPublic.EPCThread = new Thread(new ParameterizedThreadStart(PublicFunction.WriteDataThread_Timer));

                DemoPublic.EPCThread.IsBackground = true;

                DemoPublic.EPCThread.Start(obj);
            }
            else
            {
                timer1.Enabled = false;
                DemoPublic.LoopEnable = false;

                BtWriteData.Text = "Write USER";
                AddToShow("Stop to write USER");

                if (DemoPublic.EPCThread != null)
                {
                    DemoPublic.EPCThread.Abort();
                }
            }

        }

        private void BtConfigure_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            Settings settings = new Settings();

            settings.Show();
        }

        private void BtBase_Click(object sender, EventArgs e)
        {
            if (!DemoPublic.Enabel_flg)
            {
                MessageBox.Show("Please connect a device");
                return;
            }

            string infoStr = "";
            byte[] bSerial = new byte[6];
            byte[] bVersion = new byte[3];
            if (PublicFunction.GetVersion(bSerial, bVersion))
            {
                infoStr += "Hardware serial number is " + DemoPublic.BytesToHexString(bSerial, 6) + "\n";
                infoStr += "Firmware serial number is " + bVersion[0].ToString() + "." + bVersion[1].ToString() + "." + bVersion[2].ToString() + "\n";
            }
            else
            {
                infoStr += "Fail to read hardware serial number and firmware serial number\n";
            }

            MessageBox.Show(infoStr, "Product Information");
        }

        private void BtClear_Click(object sender, EventArgs e)
        {
            DataGVTags.Rows.Clear();
            DataGVTags.Refresh();
            DemoPublic.tagList.Clear();
            DemoPublic.uiiList.Clear();
            DemoPublic.CntToZero();
            TbTagCnt.Text = "0";
            TbShow.Text = "";
        }

        private void BtSave_Click(object sender, EventArgs e)
        {
            AddToShow("");

            string stFilePath = Application.StartupPath.Trim() + "\\Record-" + DateTime.Now.ToString("yyyy-MM-dd-hh-mm-ss") + ".txt";
            StreamWriter swStream;
            string str = "";

            if (File.Exists(stFilePath))
            {
                swStream = new StreamWriter(stFilePath);
            }
            else
            {
                swStream = File.CreateText(stFilePath);
            }

            for (int i = 0; i < DataGVTags.Rows.Count; i++)
            {
                for (int j = 0; j < 10; j++)
                {
                    string _strTemp = DataGVTags.Rows[i].Cells[j].Value.ToString();
                    str += _strTemp + "\0";
                }

                swStream.WriteLine(str);
                str = "";
            }

            //关闭流,释放资源
            swStream.Flush();
            swStream.Close();
            //导入Txt文件后,自动打开文件
            System.Diagnostics.Process.Start("notepad.exe", stFilePath);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            DemoPublic.TagThread tagThread = new DemoPublic.TagThread(Public.DemoPublic.ShowLoop);
            DemoPublic.PublicDM.BeginInvoke(tagThread);
        }

        private void TbData_TextChanged(object sender, EventArgs e)
        {
            if (RbHex.Checked)
            {
                string tempStr = TbData.Text.ToString().Trim().Replace(" ", "");

                string resStr = DataCheck(tempStr);

                TbData.Text = resStr;

                TbData.SelectionStart = resStr.Length;
            }
            else
            {
                string tempStr = TbData.Text.ToString().Trim().Replace(" ", "");
                string resStr = "";

                for (int i = 0; i < tempStr.Length; i++)
                {
                    resStr += tempStr[i] + " ";
                }

                TbData.Text = resStr;

                TbData.SelectionStart = resStr.Length;
            }
        }

        private void TbData_Leave(object sender, EventArgs e)
        {
            if (RbHex.Checked)
            {
                string tempStr = TbData.Text.ToString().Trim().Replace(" ", "");
                string resStr = DataCheck(tempStr);
                if (tempStr.Length % 4 == 1)
                {
                    resStr = tempStr + "000";
                }
                else if (tempStr.Length % 4 == 2)
                {
                    resStr = tempStr + "00";
                }
                else if (tempStr.Length % 4 == 3)
                {
                    resStr = tempStr + "0";
                }

                TbData.Text = resStr;

                TbData.SelectionStart = resStr.Length;
            }
        }

        private void TbData_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 0x08)
            {
                string tempStr = TbData.Text.ToString();

                int len = tempStr.Length;

                if (len > 0 && tempStr.Substring(len - 1) == " ")
                {
                    tempStr = tempStr.Substring(0, len - 1);

                    TbData.Text = tempStr;

                    TbData.SelectionStart = tempStr.Length;
                }
            }
        }

        private string DataCheck(string str)
        {
            string tempStr = "";
            string resStr = "";

            char[] c = str.ToUpper().ToCharArray();
            for (int i = 0; i < c.Length; i++)
            {
                if (System.Text.RegularExpressions.Regex.IsMatch((c[i]).ToString(), "[0-9]") || System.Text.RegularExpressions.Regex.IsMatch((c[i]).ToString(), "[A-F]"))
                {
                    tempStr += c[i];
                }
            }

            for (int i = 0; i < tempStr.Length; i++)
            {
                resStr += tempStr[i];
                if ((i + 1) % 4 == 0)
                {
                    resStr += " ";
                }
            }

            return resStr;
        }

        public void GetComList()
        {
            RegistryKey keyCom = Registry.LocalMachine.OpenSubKey("Hardware\\DeviceMap\\SerialComm");
            if (keyCom != null)
            {
                string[] sSubKeys = keyCom.GetValueNames();
                this.CbCom.Items.Clear();
                foreach (string sName in sSubKeys)
                {
                    string sValue = (string)keyCom.GetValue(sName);
                    this.CbCom.Items.Add(sValue);
                }
            }
        }

        public void AddToShow(string str)
        {
            TbShow.AppendText(str + "\r\n");
        }

        private void DataGVTags_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            /*try
            {
                if (e.RowIndex >= 0)
                {
                    TbUii.Text = DataGVTags.Rows[e.RowIndex].Cells[2].Value.ToString();

                    if (DataGVTags.CurrentCellAddress.X == 10)
                    {
                        if (!(bool)DataGVTags.Rows[e.RowIndex].Cells[10].EditedFormattedValue)
                        {
                            string userStr = "";
                            string user_Str = DataGVTags.Rows[e.RowIndex].Cells[6].Value.ToString().Trim();

                            if (user_Str != "")
                            {
                                userStr = DemoPublic.UnicodeHexStringToHanzi(user_Str);

                                DataGVTags.Rows[e.RowIndex].Cells[10].Value = false;
                                DataGVTags.Rows[e.RowIndex].Cells[6].Selected = true;
                                MessageBox.Show("" + userStr);

                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {

            }*/

            try
            {
                long startTime = Environment.TickCount;

                if (e.RowIndex >= 0)
                {
                    TbUii.Text = DataGVTags.Rows[e.RowIndex].Cells[2].Value.ToString();

                    if (DataGVTags.CurrentCellAddress.X == 10)
                    {
                        if (!(bool)DataGVTags.Rows[e.RowIndex].Cells[10].EditedFormattedValue)
                        {
                            string userStr = "";
                            string user_Str = DataGVTags.Rows[e.RowIndex].Cells[6].Value.ToString().Trim();

                            if (user_Str != "")
                            {
                                userStr = DemoPublic.UnicodeHexStringToHanzi(user_Str);
                                DataGVTags.Rows[e.RowIndex].Cells[6].Value = userStr;
                            }
                        }
                        else
                        {
                            string userStr = "";
                            string user_Str = DataGVTags.Rows[e.RowIndex].Cells[6].Value.ToString().Trim();

                            if (user_Str != "")
                            {
                                userStr = DemoPublic.HanziToUnicodeHexString(user_Str);
                                DataGVTags.Rows[e.RowIndex].Cells[6].Value = userStr;
                            }
                        }
                    }
                }

                Thread.Sleep(1);
                long timeSpan = Environment.TickCount - startTime;
                Console.WriteLine("TimeSpan = " + timeSpan);
            }
            catch (Exception ex)
            {

            }
        }

        private void RbHanzi_CheckedChanged(object sender, EventArgs e)
        {
            if (RbHanzi.Checked)
            {
                string hexStr = TbData.Text.ToString().Trim().Replace(" ", "");

                string hanziStr = DemoPublic.UnicodeHexStringToHanzi(hexStr);

                TbData.Text = hanziStr;
            }
        }

        private void RbHex_CheckedChanged(object sender, EventArgs e)
        {
            if (RbHex.Checked)
            {
                string hanziStr = TbData.Text.ToString().Trim().Replace(" ", "");

                string hexStr = DemoPublic.HanziToUnicodeHexString(hanziStr);

                TbData.Text = hexStr;
            }
        }

        private void Main_Resize(object sender, EventArgs e)
        {
            DataGVTags.Width = this.Width - groupBox1.Width - 40;
            DataGVTags.Height = this.Height - 55;
            groupBox1.Left = this.Width - groupBox1.Width - 20;
            groupBox1.Height = this.Height - 55;
            TbShow.Height = this.Height - TbTagCnt.Bottom - 67;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            byte[] power = BitConverter.GetBytes(int.Parse(txtpower.Text));
            if (DemoPublic.UhfSetPower(DemoPublic.hCom, 0x01, power[0], 1) == true)
            {
                TbShow.Text = ("Set Power OK");
            }
            else
            {
                TbShow.Text = ("Set Power Fail");

            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            byte[] power = new byte[1];
            if (DemoPublic.UhfGetPower(DemoPublic.hCom, power, 1) == true)
            {
                txtpower.Text = (power[0].ToString());
                TbShow.Text = "Get power OK ";
            }
            else
            {
                TbShow.Text = "Get power fail";
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            byte[] pwd = StrToByteArray(txtpwd.Text);
            byte[] ptr = { 0 };
            byte[] readdata = new byte[16];
            byte[] uii = new byte[16];
            byte[] uiilen = new byte[1];
            byte[] Erorr = new byte[1];
            byte[] writedata = StrToByteArray(txtEpc.Text);
            byte[] uStatus = new byte[1];
            byte[] uWritedLen = new byte[1];
            byte[] uUii = new byte[16];
            byte ucnt = Convert.ToByte(int.Parse(txtcnt.Text));
            if (DemoPublic.UhfReadDataFromSingleTag(DemoPublic.hCom, pwd, 0x01, ptr, 0x06, readdata, uii, uiilen, Erorr, 1) == true)
            {
                if (DemoPublic.UhfBlockWriteEPCByEPC(DemoPublic.hCom, pwd, ucnt, uii, writedata, Erorr, uStatus, uWritedLen, uUii, 0) == true)
                {
                    MessageBox.Show("Write OK" + "\n" + BitConverter.ToString(uUii));

                }
                else
                {
                    MessageBox.Show("write fail ");

                }
            }
            else
            {
                MessageBox.Show("Read tag fail");
            }
        }

        private void label9_Click(object sender, EventArgs e)
        {

        }

        private void button3_Click(object sender, EventArgs e)
        {
            byte[] pwd = { 0x00, 0x00, 0x00, 0x00 };
            byte[] ptr = { 0 };
            byte[] readdata = new byte[16];
            byte[] uii = new byte[16];
            byte[] uiilen = new byte[1];
            byte[] Erorr = new byte[1];

            if (DemoPublic.UhfReadDataFromSingleTag(DemoPublic.hCom, pwd, 0x01, ptr, 0x08, readdata, uii, uiilen, Erorr, 1) == true)
            {
                MessageBox.Show(BitConverter.ToString(uii));
                byte[] pwd2 = BitConverter.GetBytes(Convert.ToInt64(txtkill.Text, 16));

                if (DemoPublic.UhfKillTagByEPC(DemoPublic.hCom, pwd2, uii, Erorr, 0) == true)
                {
                    MessageBox.Show("kill tag ok ");
                }
                else
                {
                    MessageBox.Show("kill tag fail " + Erorr[0].ToString());
                }
            }


            else
            {
                MessageBox.Show("no");
            }
        }


        public static byte[] StrToByteArray(string str)
        {
            Dictionary<string, byte> hexindex = new Dictionary<string, byte>();
            for (int i = 0; i <= 255; i++)
                hexindex.Add(i.ToString("x2"), (byte)i);

            List<byte> hexres = new List<byte>();
            for (int i = 0; i < str.Length; i += 2)
                hexres.Add(hexindex[str.Substring(i, 2)]);

            return hexres.ToArray();
        }




        private void button6_Click(object sender, EventArgs e)
        {
            byte[] pwd = StrToByteArray(txtaccesspwd.Text);
            byte[] ptr = { 0 };
            byte[] readdata = new byte[8];
            byte[] uii = new byte[16];
            byte[] uiilen = new byte[1];
            byte[] Erorr = new byte[1];
            byte[] writedata = StrToByteArray(txtwritekillpwd.Text);
            byte[] uStatus = new byte[1];
            byte[] pwd2 = StrToByteArray(txtwritekillpwd.Text);
            byte[] writelen = new byte[1];

            if (DemoPublic.UhfReadDataFromSingleTag(DemoPublic.hCom, pwd, 0x00, ptr, 0x04, readdata, uii, uiilen, Erorr, 1) == true)
            {

                if (DemoPublic.UhfBlockWriteDataToSingleTag(DemoPublic.hCom, pwd, 0x00, ptr, 0x02, writedata, uii, uiilen, uStatus, Erorr, writelen, 0) == true)
                {
                    MessageBox.Show("write OK" + "\n" + BitConverter.ToString(readdata));
                }
                else
                {
                    MessageBox.Show("write fail ");

                }
            }
            else
            {
                MessageBox.Show("no");
            }


        }

        private void label5_Click(object sender, EventArgs e)
        {

        }

        private void button7_Click(object sender, EventArgs e)
        {

            byte[] pwd = StrToByteArray(txtPwd2.Text);
            byte[] ptr = { 0 };
            byte[] readdata = new byte[8];
            byte[] uii = new byte[16];
            byte[] uiilen = new byte[1];
            byte[] Erorr = new byte[1];
            byte[] writedata = StrToByteArray(txtwriteaccess.Text);
            byte[] uStatus = new byte[1];
            byte[] pwd2 = StrToByteArray(txtwritekillpwd.Text);
            byte[] writelen = new byte[1];
            byte[] ptr2 = { 2 };

            if (DemoPublic.UhfReadDataFromSingleTag(DemoPublic.hCom, pwd, 0x00, ptr, 0x04, readdata, uii, uiilen, Erorr, 1) == true)
            {

                if (DemoPublic.UhfBlockWriteDataToSingleTag(DemoPublic.hCom, pwd, 0x00, ptr2, 0x02, writedata, uii, uiilen, uStatus, Erorr, writelen, 0) == true)
                {
                    MessageBox.Show("write OK" + "\n" + BitConverter.ToString(readdata));
                }
                else
                {
                    MessageBox.Show("write fail ");

                }
            }
            else
            {
                MessageBox.Show("no");
            }
        }

        private void label14_Click(object sender, EventArgs e)
        {

        }

        private void button8_Click(object sender, EventArgs e)
        {

            if (DemoPublic.UhfStartInventory(DemoPublic.hCom, 1, 1, 0) == true)
            {
                TbShow.Text = "inventory start";
                timer2.Enabled = true;

            }
            else
            {
                TbShow.Text = "inventory start fail";
            }

        }

        private void button9_Click(object sender, EventArgs e)
        {
            DemoPublic.UhfStopOperation(DemoPublic.hCom, 0);
            timer2.Enabled = false;
        }
        byte[] hid = new byte[14];
        private void timer2_Tick(object sender, EventArgs e)
        {
            byte[] ulen = new byte[1];
            byte[] uii = new byte[14];

            if (DemoPublic.UhfReadInventory(DemoPublic.hCom, ulen, uii) == true)
            {
                if (BitConverter.ToString(uii) != BitConverter.ToString(hid))
                {
                    hid = uii;
                    listBox1.Items.Add(BitConverter.ToString(uii));
                    listBox1.SelectedIndex = listBox1.Items.Count - 1;
                    UHFReader.HID.WriteChar(BitConverter.ToString(uii));

                }

            }

        }

        private void button10_Click(object sender, EventArgs e)
        {

        }

        private void button10_Click_1(object sender, EventArgs e)
        {
            byte[] aaa = new byte[17];
            if (DemoPublic.LockGenCode(4, 1, 2, 4, 4, aaa) == true)
            {
                MessageBox.Show("ok");
                MessageBox.Show(BitConverter.ToString(aaa));

            }
            else
            {
                MessageBox.Show("no");
            }
        }

        private void txtwritekillpwd_TextChanged(object sender, EventArgs e)
        {

        }
        
        private void button5_Click(object sender, EventArgs e)
        {

        }
        byte[] pwd = { 0x11, 0x11, 0x11, 0x11};
        byte[] uUii = new byte[16];
        byte[] bbbb = { 0x01 };
        private void button5_Click_1(object sender, EventArgs e)
        {
            byte[] aaa = new byte[3];
            if (DemoPublic.LockGenCode(1, 1, 1, 1, 1, aaa) == true)
            {
                MessageBox.Show("ok" + BitConverter.ToString(aaa));
            }
            else
            {
                MessageBox.Show("no");
            }
            //byte[] pwd = StrToByteArray(txtLockTag.Text);           
            byte[] error = new byte[1];
          

            if (DemoPublic.UhfLockMemFromSingleTag(DemoPublic.hCom, pwd,aaa , uUii, error, 0) == true)
            {
                
                MessageBox.Show("lock ok " + BitConverter.ToString(error));
            }
            else
            {
                byte[] error2 = new byte[1];
                MessageBox.Show("lock no " + BitConverter.ToString(error));
            }

        }

        private void button11_Click(object sender, EventArgs e)
        {
            

            byte[] pwd = { 0x00, 0x00, 0x00, 0x00  };
            byte[] ptr = { 2 };
            byte[] readdata = new byte[8];
            byte[] uii = new byte[16];
            byte[] payload = new byte[6];
            payload[0] = DemoPublic.bQ;
            payload[1] = 0x20;
            if (DemoPublic.UhfStartReadDataFromMultiTag(DemoPublic.hCom, pwd, 0x02, ptr, 0x08, 0, payload , 0) == true)
            {
                timer3.Enabled = true;
            }
            else
            {
                MessageBox.Show("no");
              
            }
        }

        private void CbQ_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void timer3_Tick(object sender, EventArgs e)
        {
            byte[] uStatus = new byte[1];
            byte[] ufData_len = new byte[1];
            byte[] ufReadData = new byte[18];
            byte[] usData_len = new byte[1];
            byte[] usReadData = new byte[1];
            byte[] uUii = new byte[16];
            byte[] uLenUii = new byte[1];
            byte[] uErrorCode = new byte[1];
            DemoPublic.UhfGetDataFromMultiTag(DemoPublic.hCom, uStatus, ufData_len, ufReadData, usData_len, usReadData, uUii, uLenUii);
        }

        private void button12_Click(object sender, EventArgs e)
        {

            byte[] pwd = { 0x00, 0x00, 0x00, 0x00 };
            byte bank = 0x02;
            byte[] ptr = { 0x01 };
            byte cnt = 0x01;
            byte[] payload = new byte[6];
            byte option=0;
            int ptr_ebv = 0;

            
            if (option  == 1)
            {
                payload[0] = 0x02;


                payload[1] = 0x01;
                    payload[2] = 0x01;

                payload[2 + ptr_ebv] = 0x01;
                payload[3 + ptr_ebv] = DemoPublic.bQ; ;
                payload[4 + ptr_ebv] = 0x20;
            }
            else
            {
                payload[0] = DemoPublic.bQ; ;
                payload[1] = 0x20;
            }

            //   if (DemoPublic.UhfStartReadDataFromMultiTag(DemoPublic.hCom, pwd, bank, ptr, cnt, option, payload, DemoPublic.flagCrc)==true )
            MessageBox.Show((DemoPublic.UhfStartReadDataFromMultiTag(DemoPublic.hCom, pwd, bank, ptr, cnt, option, payload, DemoPublic.flagCrc).ToString()));
           // else
           //     MessageBox.Show("no");


        }
    }
    }
